@@ -13,6 +13,7 @@ from abc import ABCMeta, abstractmethod
 from django.contrib.admindocs.utils import trim_docstring
 
 from rest_framework.views import get_view_name, get_view_description
+from rest_framework import viewsets
 from rest_framework.compat import apply_markdown, smart_text
 from rest_framework.utils import formatting
 
@@ -62,7 +63,7 @@ class IntrospectorHelper(object):
         if cut_off is not None:
             split_lines = split_lines[0:cut_off]
 
-        return "<br/>".join(split_lines)
+        return "\n".join(split_lines)
 
     @staticmethod
     def get_serializer_name(serializer):
@@ -79,7 +80,11 @@ class IntrospectorHelper(object):
         """
         Returns the first sentence of the first line of the class docstring
         """
-        return get_view_description(callback).split("\n")[0].split(".")[0]
+        from .compat import strip_tags
+        description = get_view_description(callback).split("\n")[0].split(".")[0]
+        if apply_markdown:
+            description = strip_tags(do_markdown(description))
+        return description
 
 
 class BaseViewIntrospector(object):
@@ -103,7 +108,12 @@ class BaseViewIntrospector(object):
 
     def get_serializer_class(self):
         if hasattr(self.callback, 'get_serializer_class'):
-            return self.callback().get_serializer_class()
+            view = self.callback()
+            if not hasattr(view, 'kwargs'):
+                view.kwargs = dict()
+            if hasattr(self.pattern, 'default_args'):
+                view.kwargs.update(self.pattern.default_args)
+            return view.get_serializer_class()
 
     def get_description(self):
         """
@@ -161,13 +171,30 @@ class BaseMethodIntrospector(object):
             serializer = self.parent.get_serializer_class()
         return serializer
 
+    def get_response_serializer_class(self):
+        parser = self.get_yaml_parser()
+        serializer = parser.get_response_serializer_class(self.callback)
+        if serializer is None:
+            serializer = self.get_serializer_class()
+        return serializer
+
+    def get_request_serializer_class(self):
+        parser = self.get_yaml_parser()
+        serializer = parser.get_request_serializer_class(self.callback)
+        if serializer is None:
+            serializer = self.get_serializer_class()
+        return serializer
+
     def get_summary(self):
         docs = self.get_docs()
 
         # If there is no docstring on the method, get class docs
         if docs is None:
             docs = self.parent.get_description()
-        docs = trim_docstring(docs).split('\n')[0]
+        docs = trim_docstring(docs).split("\n")[0].split(".")[0]
+        if apply_markdown:
+            from .compat import strip_tags
+            docs = strip_tags(do_markdown(docs))
 
         return docs
 
@@ -201,13 +228,7 @@ class BaseMethodIntrospector(object):
             )
             docstring += '\n' + method_docs
 
-        # Markdown is optional
-        if apply_markdown:
-            docstring = apply_markdown(docstring)
-        else:
-            docstring = docstring.replace("\n\n", "<br/>")
-
-        return docstring
+        return do_markdown(docstring)
 
     def get_parameters(self):
         """
@@ -253,7 +274,7 @@ class BaseMethodIntrospector(object):
         return getattr(self.callback, method).__doc__
 
     def build_body_parameters(self):
-        serializer = self.get_serializer_class()
+        serializer = self.get_request_serializer_class()
         serializer_name = IntrospectorHelper.get_serializer_name(serializer)
 
         if serializer_name is None:
@@ -308,7 +329,7 @@ class BaseMethodIntrospector(object):
         Builds form parameters from the serializer class
         """
         data = []
-        serializer = self.get_serializer_class()
+        serializer = self.get_request_serializer_class()
 
         if serializer is None:
             return data
@@ -378,7 +399,15 @@ class WrappedAPIViewIntrospector(BaseViewIntrospector):
         class_docs = smart_text(class_docs)
         class_docs = IntrospectorHelper.strip_yaml_from_docstring(class_docs)
         class_docs = IntrospectorHelper.strip_params_from_docstring(class_docs)
-        return class_docs
+        return do_markdown(class_docs)
+
+
+def do_markdown(docstring):
+    # Markdown is optional
+    if apply_markdown:
+        return apply_markdown(docstring)
+    else:
+        return docstring.replace("\n\n", "<br/>")
 
 
 class APIViewMethodIntrospector(BaseMethodIntrospector):
@@ -418,6 +447,8 @@ class ViewSetIntrospector(BaseViewIntrospector):
 
     def __init__(self, callback, path, pattern, patterns=None):
         super(ViewSetIntrospector, self).__init__(callback, path, pattern)
+        if not issubclass(callback, viewsets.ViewSetMixin):
+            raise Exception("wrong callback passed to ViewSetIntrospector")
         self.patterns = patterns or [pattern]
 
     def __iter__(self):
@@ -720,6 +751,28 @@ class YAMLDocstringParser(object):
             pass
         return None
 
+    def get_request_serializer_class(self, callback):
+        """
+        Retrieves request serializer class from YAML object
+        """
+        serializer = self.object.get('request_serializer', None)
+        try:
+            return self._load_class(serializer, callback)
+        except (ImportError, ValueError):
+            pass
+        return None
+
+    def get_response_serializer_class(self, callback):
+        """
+        Retrieves response serializer class from YAML object
+        """
+        serializer = self.object.get('response_serializer', None)
+        try:
+            return self._load_class(serializer, callback)
+        except (ImportError, ValueError):
+            pass
+        return None
+
     def get_response_type(self):
         """
         Docstring may define custom response class
@@ -791,10 +844,10 @@ class YAMLDocstringParser(object):
 
             # Min/Max are optional
             if 'minimum' in field and data_type == 'integer':
-                f['minimum'] = field.get('minimum', 0)
+                f['minimum'] = str(field.get('minimum', 0))
 
             if 'maximum' in field and data_type == 'integer':
-                f['maximum'] = field.get('maximum', 0)
+                f['maximum'] = str(field.get('maximum', 0))
 
             # enum options
             enum = field.get('enum', [])
