@@ -1,6 +1,6 @@
 import datetime
 
-from django.core.urlresolvers import RegexURLResolver
+from django.core.urlresolvers import RegexURLResolver, RegexURLPattern
 from django.conf import settings
 from django.conf.urls import patterns, url, include
 from django.contrib.auth.models import User
@@ -25,6 +25,18 @@ from .introspectors import ViewSetIntrospector, APIViewIntrospector, \
     IntrospectorHelper, APIViewMethodIntrospector
 
 
+def no_markdown(func):
+    def func_sans_markdown(*args, **kwargs):
+        import rest_framework.compat
+        apply_markdown = rest_framework.compat.apply_markdown
+        try:
+            rest_framework.compat.apply_markdown = None
+            func(*args, **kwargs)
+        finally:
+            rest_framework.compat.apply_markdown = apply_markdown
+    return func_sans_markdown
+
+
 class MockApiView(APIView):
     """
     A Test View
@@ -49,6 +61,10 @@ class CommentSerializer(serializers.Serializer):
     created = serializers.DateTimeField(default=datetime.datetime.now)
 
 
+class QuerySerializer(serializers.Serializer):
+    query = serializers.CharField(max_length=100)
+
+
 class UrlParserTest(TestCase):
     def setUp(self):
         self.url_patterns = patterns(
@@ -324,6 +340,20 @@ class DocumentationGeneratorTest(TestCase):
         delta = datetime.timedelta(seconds=1)
         self.assertAlmostEqual(value, datetime.datetime.now(), delta=delta)
 
+    def test_get_models_ordering(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+        urlparser = UrlParser()
+        url_patterns = patterns('', url(r'my-api/', SerializedAPI.as_view()))
+        apis = urlparser.get_apis(url_patterns)
+
+        docgen = DocumentationGenerator()
+        models = docgen.get_models(apis)
+
+        self.assertIn('CommentSerializer', models)
+        self.assertEqual(list(models['CommentSerializer']['properties'].keys()), ["email", "content", "created"])
+
     def test_get_serializer_set(self):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
@@ -364,6 +394,20 @@ class DocumentationGeneratorTest(TestCase):
 
         self.assertIs(serializer_class, CommentSerializer)
 
+    def test_nested_serializer(self):
+        class ASerializer(serializers.Serializer):
+            point = CommentSerializer()
+            query = QuerySerializer()
+
+        docgen = DocumentationGenerator()
+        serializerses = docgen._find_field_serializers([ASerializer])
+        self.assertTrue([s for s in serializerses
+                         if isinstance(s, CommentSerializer)])
+        self.assertTrue([s for s in serializerses
+                         if isinstance(s, QuerySerializer)])
+        self.assertFalse([s for s in serializerses
+                         if isinstance(s, ASerializer)])
+
 
 class IntrospectorHelperTest(TestCase):
     def test_strip_yaml_from_docstring(self):
@@ -456,6 +500,49 @@ class BaseViewIntrospectorTest(TestCase):
         introspector = APIViewIntrospector(MockApiView, '/', RegexURLResolver(r'^/', ''))
         self.assertEqual(None, introspector.get_serializer_class())
 
+    def test_get_serializer_class_url_kwargs(self):
+        class SerializerFoo(serializers.Serializer):
+            pass
+
+        class SerializerBar(serializers.Serializer):
+            pass
+
+        class TestView(APIView):
+            serializer_class = SerializerFoo
+
+            def get_serializer_class(self):
+                if 'b' in self.kwargs:
+                    return SerializerFoo
+                return SerializerBar
+        introspector = APIViewIntrospector(TestView, '/a', RegexURLPattern(r'^a/$', '', {'b': True}))
+        self.assertEqual(SerializerFoo, introspector.get_serializer_class())
+        introspector = APIViewIntrospector(TestView, '/a', RegexURLPattern(r'^a/$', ''))
+        self.assertEqual(SerializerBar, introspector.get_serializer_class())
+
+        urlparser = UrlParser()
+        url_patterns = patterns(
+            '',
+            url(r'^a/$', TestView.as_view()),
+        )
+        apis = urlparser.get_apis(url_patterns)
+
+        docgen = DocumentationGenerator()
+        serializer_set = docgen._get_serializer_set(apis)
+        self.assertEqual(1, len(serializer_set))
+        self.assertEqual(SerializerBar, list(serializer_set)[0])
+
+        urlparser = UrlParser()
+        url_patterns = patterns(
+            '',
+            url(r'^a/b$', TestView.as_view(), {'b': True}),
+        )
+        apis = urlparser.get_apis(url_patterns)
+
+        docgen = DocumentationGenerator()
+        serializer_set = docgen._get_serializer_set(apis)
+        self.assertEqual(1, len(serializer_set))
+        self.assertEqual(SerializerFoo, list(serializer_set)[0])
+
 
 class BaseMethodIntrospectorTest(TestCase):
     def test_get_method_docs(self):
@@ -467,7 +554,7 @@ class BaseMethodIntrospectorTest(TestCase):
                 """
             pass
 
-        class_introspector = ViewSetIntrospector(TestApiView, '/{pk}', RegexURLResolver(r'^/(?P<{pk}>[^/]+)$', ''))
+        class_introspector = APIViewIntrospector(TestApiView, '/{pk}', RegexURLResolver(r'^/(?P<{pk}>[^/]+)$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'GET')
         docs_get = introspector.get_docs()
 
@@ -481,7 +568,7 @@ class BaseMethodIntrospectorTest(TestCase):
             """
             pass
 
-        class_introspector = ViewSetIntrospector(MyListView, '/{pk}', RegexURLResolver(r'^/(?P<{pk}>[^/]+)$', ''))
+        class_introspector = APIViewIntrospector(MyListView, '/{pk}', RegexURLResolver(r'^/(?P<{pk}>[^/]+)$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         method_docs = introspector.get_summary()
 
@@ -491,7 +578,7 @@ class BaseMethodIntrospectorTest(TestCase):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
 
-        class_introspector = ViewSetIntrospector(SerializedAPI, '/', RegexURLResolver(r'^/$', ''))
+        class_introspector = APIViewIntrospector(SerializedAPI, '/', RegexURLResolver(r'^/$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         params = introspector.build_body_parameters()
 
@@ -501,7 +588,7 @@ class BaseMethodIntrospectorTest(TestCase):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
 
-        class_introspector = ViewSetIntrospector(SerializedAPI, '/', RegexURLResolver(r'^/$', ''))
+        class_introspector = APIViewIntrospector(SerializedAPI, '/', RegexURLResolver(r'^/$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         params = introspector.build_form_parameters()
 
@@ -516,7 +603,7 @@ class BaseMethodIntrospectorTest(TestCase):
         class MyAPIView(ListCreateAPIView):
             serializer_class = MySerializer
 
-        class_introspector = ViewSetIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
+        class_introspector = APIViewIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         params = introspector.build_form_parameters()
 
@@ -536,7 +623,7 @@ class BaseMethodIntrospectorTest(TestCase):
         class MyAPIView(ListCreateAPIView):
             serializer_class = MySerializer
 
-        class_introspector = ViewSetIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
+        class_introspector = APIViewIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         params = introspector.build_form_parameters()
 
@@ -564,7 +651,7 @@ class BaseMethodIntrospectorTest(TestCase):
 
         class MyAPIView(ListCreateAPIView):
             serializer_class = MySerializer
-        class_introspector = ViewSetIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
+        class_introspector = APIViewIntrospector(MyAPIView, '/', RegexURLResolver(r'^/$', ''))
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
         params = introspector.build_form_parameters()
         self.assertEqual(2, len(params))
@@ -589,7 +676,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 pass
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             AnAPIView, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'GET')
@@ -612,7 +699,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 pass
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             AnAPIView, '/', RegexURLResolver(r'^/$', '')
         )
         class_introspector.methods = lambda: ['GET']
@@ -635,7 +722,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 pass
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             AnAPIView, '/', RegexURLResolver(r'^/$', '')
         )
         class_introspector.methods = lambda: ['GET']
@@ -723,7 +810,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -749,7 +836,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -776,7 +863,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -807,7 +894,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'GET')
@@ -822,6 +909,42 @@ class YAMLDocstringParserTests(TestCase):
         self.assertEqual(1, len(list(query_params)))
         self.assertEqual(1, len(list(form_params)))
 
+    def test_parameters_minimum_is_string(self):
+        '''
+        minimum and maximum of Parameter Object required by Swagger 1.2 spec to be string.
+        '''
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                My post view with custom post parameters
+
+                ---
+                parameters_strategy:
+                    form: replace
+                    query: replace
+                parameters:
+                    - name: name
+                      type: integer
+                      required: true
+                      minimum: 1
+                      maximum: 100
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        parser = introspector.get_yaml_parser()
+        params = parser.discover_parameters(introspector)
+        self.assertEqual(len(params), 1)
+        self.assertIn('minimum', params[0])
+        self.assertEqual(params[0]['minimum'], '1')
+        self.assertIn('maximum', params[0])
+        self.assertEqual(params[0]['maximum'], '100')
+
     def test_response_messages(self):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
@@ -837,7 +960,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -859,7 +982,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -959,7 +1082,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -967,6 +1090,291 @@ class YAMLDocstringParserTests(TestCase):
         serializer = generator._get_method_serializer(introspector)
         self.assertEqual(serializer, None)
 
+    def test_request_response_serializers1(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                ---
+                request_serializer: QuerySerializer
+                response_serializer: CommentSerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        url_patterns = patterns('', url(r'my-api/', SerializedAPI.as_view()))
+        urlparser = UrlParser()
+        apis = urlparser.get_apis(url_patterns)
+        models = generator.get_models(apis)
+        self.assertIn('SerializedAPIPostResponse', models)
+        self.assertIn('WriteCommentSerializer', models)
+        self.assertIn('CommentSerializer', models)
+        self.assertNotIn('QuerySerializer', models)
+        self.assertNotIn('WriteQuerySerializer', models)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers2(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                ---
+                serializer: QuerySerializer
+                response_serializer: CommentSerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers3(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                ---
+                request_serializer: QuerySerializer
+                serializer: CommentSerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers4(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                ---
+                request_serializer: QuerySerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers5(self):
+        class SerializedAPI(ListCreateAPIView):
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                ---
+                response_serializer: CommentSerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers5_1(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    request_serializer: QuerySerializer
+                    response_serializer: CommentSerializer
+            """
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers6(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    serializer: QuerySerializer
+                    response_serializer: CommentSerializer
+            """
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers7(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    request_serializer: QuerySerializer
+                    serializer: CommentSerializer
+            """
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers8(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    request_serializer: QuerySerializer
+            """
+            serializer_class = CommentSerializer
+
+            def post(self, request, *args, **kwargs):
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers9(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    response_serializer: CommentSerializer
+            """
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
+    def test_request_response_serializers10(self):
+        class SerializedAPI(ListCreateAPIView):
+            """
+                ---
+                POST:
+                    request_serializer: CommentSerializer
+                    response_serializer: QuerySerializer
+            """
+            serializer_class = QuerySerializer
+
+            def post(self, request, *args, **kwargs):
+                """
+                    ---
+                    request_serializer: QuerySerializer
+                    response_serializer: CommentSerializer
+                """
+                return super(SerializedAPI, self).post(request, *args, **kwargs)
+        class_introspector = APIViewIntrospector(
+            SerializedAPI, '/', RegexURLResolver(r'^/$', '')
+        )
+        introspector = APIViewMethodIntrospector(class_introspector, 'POST')
+        generator = DocumentationGenerator()
+        serializer = generator._get_method_serializer(introspector)
+        self.assertEqual(serializer, CommentSerializer)
+        body_parameters = introspector.build_body_parameters()
+        form_parameters = introspector.build_form_parameters()
+        self.assertEqual(body_parameters['name'], "QuerySerializer")
+        self.assertEqual(len(form_parameters), 1)
+        self.assertEqual(form_parameters[0]['name'], "query")
+
     def test_custom_response_class(self):
         class SerializedAPI(ListCreateAPIView):
             serializer_class = CommentSerializer
@@ -984,7 +1392,7 @@ class YAMLDocstringParserTests(TestCase):
                 """
                 return super(SerializedAPI, self).post(request, *args, **kwargs)
 
-        class_introspector = ViewSetIntrospector(
+        class_introspector = APIViewIntrospector(
             SerializedAPI, '/', RegexURLResolver(r'^/$', '')
         )
         introspector = APIViewMethodIntrospector(class_introspector, 'POST')
@@ -999,6 +1407,7 @@ class YAMLDocstringParserTests(TestCase):
         self.assertIn('SerializedAPIPostResponse',
                       generator.explicit_response_types)
 
+    @no_markdown
     def test_fbv_notes(self):
 
         @api_view(["POST"])
@@ -1019,3 +1428,61 @@ class YAMLDocstringParserTests(TestCase):
         notes = introspector.get_notes()
 
         self.assertEqual(notes, "Slimy toads")
+
+    def test_fbv_markdown(self):
+
+        @api_view(["POST"])
+        def a_view(request):
+            """
+            Slimy *toads*
+            """
+            return "blarg"
+
+        class_introspector = WrappedAPIViewIntrospector(
+            func_to_wrapper(a_view), '/', RegexURLResolver(r'^/$', '')
+        )
+
+        url_patterns = patterns('', url(r'my-api/', a_view))
+        urlparser = UrlParser()
+        generator = DocumentationGenerator()
+        apis = urlparser.get_apis(url_patterns)
+        notes = class_introspector.get_notes()
+        self.assertEqual(notes, "<p>Slimy <em>toads</em></p>")
+        introspector = WrappedAPIViewMethodIntrospector(class_introspector, 'POST')
+
+        notes = introspector.get_notes()
+
+        self.assertEqual(notes, "<p>Slimy <em>toads</em></p>")
+        api_docs = generator.generate(apis)
+        self.assertEqual(len(api_docs), 1)
+        self.assertIn("description", api_docs[0])
+        self.assertEqual(api_docs[0]["description"], "Slimy toads")
+        self.assertIn('operations', api_docs[0])
+        self.assertEqual(api_docs[0]["operations"][0]['summary'], "Slimy toads")
+
+    def test_apiview_models(self):
+        from rest_framework.views import Response
+
+        class MyCustomView(APIView):
+            """
+            Slimy toads
+            """
+
+            def get(self, *args, **kwargs):
+                """
+                param1 -- my param
+                """
+                return Response({'foo': 'bar'})
+
+            def post(self, request, *args, **kwargs):
+                """
+                horse -- the name of your horse
+                """
+                return Response({'horse': request.GET.get('horse')})
+
+        url_patterns = patterns('', url(r'my-api/', MyCustomView.as_view()))
+        urlparser = UrlParser()
+        apis = urlparser.get_apis(url_patterns)
+        generator = DocumentationGenerator()
+        models = generator.get_models(apis)
+        self.assertEqual(type(models), dict)
