@@ -27,11 +27,20 @@ except ImportError:
 from rest_framework.views import Response
 from rest_framework_swagger.urlparser import UrlParser
 from rest_framework_swagger.apidocview import APIDocView
-from rest_framework.renderers import JSONRenderer
 from rest_framework_swagger.docgenerator import DocumentationGenerator
 
 from rest_framework_swagger import SWAGGER_SETTINGS
 
+from rest_framework.settings import api_settings
+
+try:
+    JSONRenderer = list(filter(
+        lambda item: item.format == 'json',
+        api_settings.DEFAULT_RENDERER_CLASSES,
+    ))[0]
+except IndexError:
+    from rest_framework.renderers import JSONRenderer
+
 
 class SwaggerUIView(View):
 
@@ -40,7 +49,7 @@ class SwaggerUIView(View):
         if not self.has_permission(request):
             return self.handle_permission_denied(request)
 
-        template_name = "rest_framework_swagger/index.html"
+        template_name = SWAGGER_SETTINGS.get('template_path')
         data = {
             'swagger_settings': {
                 'discovery_url': "%sapi-docs/" % request.build_absolute_uri(),
@@ -90,7 +99,15 @@ class SwaggerResourcesView(APIDocView):
             'apiVersion': SWAGGER_SETTINGS.get('api_version', ''),
             'swaggerVersion': '1.2',
             'basePath': self.host.rstrip('/'),
-            'apis': apis
+            'apis': apis,
+            'info': SWAGGER_SETTINGS.get('info', {
+                'contact': '',
+                'description': '',
+                'license': '',
+                'licenseUrl': '',
+                'termsOfServiceUrl': '',
+                'title': '',
+            }),
         })
 
     def get_resources(self):
